/*
 * Copyright 2004 Sun Microsystems, Inc.
 *
 * Use is subject to the terms of the Sun Industry Standards Source License, 
 * a copy of which must accompany the software distribution.  The license terms are
 * available at http://www.opensource.org/licenses/sisslpl.php .
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

package com.sun.im.service.sample;

import com.sun.im.service.*;
import java.io.*;
import java.util.*;


/**
 * sample program which listens for notifications
 */
public class Alerter extends Thread implements NotificationSessionListener
{

    // sessions
    private NotificationSession _session;
    
    // contains all open subscriptions and news channels
    private Hashtable _subscriptions = new Hashtable();
    
    // keeps track of the latest answer to a yes or no question
    private boolean _shutdown = false;
    
    private String _user;
    
    //create a connection using the command line
    public Alerter(CollaborationSessionFactory fac, 
		       String server, String user, String password) throws Exception 
    {
	_user = user;
	CollaborationSession s = fac.getSession(server, user, password, this);
	_session = (NotificationSession)s.accessService(com.sun.im.service.CollaborationSessionFactory.NOTIFICATION);
    }
    
    public void onError(com.sun.im.service.CollaborationException e) {
	e.printStackTrace();
    }
    
    public void run()
    {
	
	while (!_shutdown) {
	    
	    System.out.println("Listening for notifications");

	    try {		
		String s = _reader.readLine();
		if (s == null) break;
		s = s.trim();
		if (s.equals("")) continue;
		
		// command
		StringTokenizer st = new StringTokenizer(s);
		String cmd = st.nextToken();
		String args[] = new String [ st.countTokens() ];
		int i = 0;
		while (st.hasMoreTokens()) {
		    args[i++] = st.nextToken();
		}
		
		if (cmd.equalsIgnoreCase("quit") ||
			   cmd.equalsIgnoreCase("exit") ||
			   cmd.equalsIgnoreCase(".")) {
		    _shutdown = true;
		}

	    } catch (Exception e) {
		System.out.println("Error " + e.toString());
		e.printStackTrace();
	    }
	    
	}
    
    }

    public synchronized void onMessage(Message m) 
    {
	try {
	    printMessage(m);
            String s = prompt("Hit Enter when you have read the message... ", false);

	    // send read status
	    m.sendStatus(MessageStatus.READ);

	} catch (Exception e) {
	    e.printStackTrace();
	}
    }


    //
    // STATIC CODE
    //
    
    // source to read commands from
    private static BufferedReader _reader;
    

    //login to the server    
    public static void main(String args[]) 
    {

	_reader = new BufferedReader(new InputStreamReader(System.in));

        String user, password, server;
        
        if (args.length > 0 && args[0].equals("?")) {
            System.out.println("login host username password");
            return;
        }

	// prompt server        
        if (args.length >= 1) {
            server = args[0];
        } else {
            server = prompt("Hostname [localhost:9909]\t: ",
			     false, "localhost:9909");
	}
        
        if (args.length >= 2) {
            user = args[1];
        } else {
	    String username = System.getProperty("user.name");
	    user = prompt("User name [" + username + "]\t: ",
			  false, username);
        }
        
        if (args.length >= 3) {
            password = args[2];
        } else {            
            password = prompt("Password [iplanet]\t: ", false, "iplanet");
        }
	
	try {
	    
	    CollaborationSessionFactory fac = new CollaborationSessionFactory();
	    Alerter thr = new Alerter(fac, server, user, password);
	    thr.start();
	    
	    thr.join();
	    
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    public static void printMessage(Message message) throws Exception
    {
        // display the message
	//        System.out.println("Message Received -- " + message.getMessageId());
        System.out.println("    Subject: " + message.getHeader("subject"));
        System.out.println("    Content-type: " + message.getContentType());
	String from = message.getOriginator();
	System.out.println("    From: " + from);
        String[] rcpts = message.getRecipients();
        for (int i = 0 ; i < rcpts.length ; i++) {
	    System.out.println("    To: " + rcpts[i]);
        }

        MessagePart[] parts = message.getParts();
        for (int i = 0 ; i < parts.length ; i++) {
            //System.out.println("    Part " + i);
            //System.out.println("        Content-type: " + parts[i].getContentType());
            try {
		BufferedReader br=new BufferedReader(new InputStreamReader(parts[i].getInputStream()));
		String str;
		while((str=br.readLine()) != null) System.out.println(str);
            } catch (Exception e) { 
                e.printStackTrace();
            }
        }
    }

    private static String prompt(String ask, boolean newline) {
	return prompt(ask, newline, "");
    }
          
    private static String prompt(String ask, boolean newline, String defaultValue) 
    {
        String ret = defaultValue;
        if (newline) {
	    System.out.println(ask);
	} else {
	    System.out.print(ask);
	}

	try {	      
	    String s = _reader.readLine();
	    if (s.length() > 0) ret = s;
	} catch (Exception e) {
	    e.printStackTrace();
	}

        return ret;
    }

}            
