/*
 * Copyright 2001-2004 Sun Microsystems, Inc.
 * Property of Sun Microsystems, Inc.
 *
 * Use is subject to the terms of the Sun Industry Standards Source License, 
 * a copy of which must accompany the software distribution.  The license terms are
 * available at http://www.opensource.org/licenses/sisslpl.php .
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

import com.sun.im.service.*;
import java.io.*;
import java.util.*;


public class Watcher extends Thread implements CollaborationSessionListener, PresenceInfoListener
{

    // sessions
    private PresenceSession _session;
    
    // contains all open subscriptions and news channels
    private Hashtable _subscriptions = new Hashtable();
    
    // keeps track of the latest answer to a yes or no question
    private boolean _shutdown = false;
    
    private String _user;
    
    //create a connection using the command line
    public Watcher(CollaborationSessionFactory fac, 
		   String server, String user, String password) throws Exception 
    {
	_user = user;
	CollaborationSession s = fac.getSession(server, user, password, this);
	_session = (PresenceSession)s.accessService(com.sun.im.service.CollaborationSessionFactory.PRESENCE);
    }
    
    public void onError(com.sun.im.service.CollaborationException e) {
	e.printStackTrace();
    }
    
    public void run()
    {
	
	while (!_shutdown) {
	    
	    try {
		System.out.print("Watcher> ");
		
		String s = _reader.readLine();
		if (s == null) break;
		s = s.trim();
		if (s.equals("")) continue;
		
		// command
		StringTokenizer st = new StringTokenizer(s);
		String cmd = st.nextToken();
		String args[] = new String [ st.countTokens() ];
		int i = 0;
		while (st.hasMoreTokens()) {
		    args[i++] = st.nextToken();
		}
		
		if (cmd.equalsIgnoreCase("subscribe") ||
		    cmd.startsWith("subs")) {
		    subscribe(args);
		} else if (cmd.equalsIgnoreCase("unsubscribe") ||
			   cmd.startsWith("unsu")) {
		    unsubscribe(args);
		} else if (cmd.equalsIgnoreCase("fetch")) {
		    fetch(args);
		} else if (cmd.equalsIgnoreCase("quit") ||
			   cmd.equalsIgnoreCase("exit") ||
			   cmd.equalsIgnoreCase(".")) {
		    _shutdown = true;
		}

	    } catch (Exception e) {
		System.out.println("Error " + e.toString());
		e.printStackTrace();
	    }
	    
	}
    
    }

    public synchronized void onExpiration(String url, java.util.Date retryAfter, int errorCode, String errorText) 
    {
	System.out.println("!!! " + url + " Subscription Expired: " + errorText + "[" + errorCode + "]");
    }

    public void onPresenceInfo(String pi) {}
    public void onError(String url, String pi) {}

    public void onPresenceInfo(String url, String pi, java.util.Date expires) 
    {
	try {
	    PresenceHelper ph = new PresenceHelper(pi);
	    
	    // for now: retain only one icon: the last. (todo)
	    for (Iterator i = ph.getTuples().iterator(); i.hasNext() ; ) {
		PresenceTuple t = (PresenceTuple)i.next();
		System.out.println(t.getContact() + " " + t.getStatus() + " " + t.getNote());
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    public void onError(String presentity, int errorCode, String errorText) 
    {
	System.out.println("!!! " + presentity + " - Error: " + errorText + "[" + errorCode + "]");
    }

    public void onError(int errorCode, String errorText) 
    {
	System.out.println("!!! General Error: " + errorText + "[" + errorCode + "]");
    }

    //check on other users status
    public void fetch(String args[]) throws Exception {
        if (args.length > 0 && args[0].equals("?")) {
            System.out.println("fetch [ entity ]");
            return;
        }
        String name;
        if (args.length >= 1) {
            name = args[0];
        } else {
            name = prompt("Entity: ", false);
        }
        
        _session.fetch(name, this);
    }    
        
    //check on other users status
    public void subscribe(String args[]) throws Exception {
        if (args.length > 0 && args[0].equals("?")) {
            System.out.println("subscribe [ entity ]");
            return;
        }
        String name;
        if (args.length >= 1) {
            name = args[0];
        } else {
            name = prompt("Entity: ",false);
        }
        
        _session.subscribe(name, this);
    }    

    //check on other users status
    public void unsubscribe(String args[]) throws Exception {
        if (args.length > 0 && args[0].equals("?")) {
            System.out.println("unsusbscribe [ entity ]");
            return;
        }
        String name;
        if (args.length >= 1) {
            name = args[0];
        } else {
            name = prompt("Entity: ",false);
        }
        
        _session.unsubscribe(name);
    }    

    //
    // STATIC CODE
    //
    
    // source to read commands from
    private static BufferedReader _reader;
    

    //login to the server    
    public static void main(String args[]) 
    {

	_reader = new BufferedReader(new InputStreamReader(System.in));

        String user, password, server;
        
        if (args.length > 0 && args[0].equals("?")) {
            System.out.println("login host username password");
            return;
        }

	// prompt server        
        if (args.length >= 1) {
            server = args[0];
        } else {
            server = prompt("Hostname [localhost:9909]\t: ",
			     false, "localhost:9909");
	}
        
        if (args.length >= 2) {
            user = args[1];
        } else {
	    String username = System.getProperty("user.name");
	    user = prompt("User name [" + username + "]\t: ",
			  false, username);
        }
        
        if (args.length >= 3) {
            password = args[2];
        } else {            
            password = prompt("Password [iplanet]\t: ", false, "iplanet");
        }
	
	try {
	    
	    CollaborationSessionFactory fac = new CollaborationSessionFactory();
	    Watcher watcher = new Watcher(fac, server, user, password);
	    watcher.start();
	    
	    watcher.join();
	    
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    private static String prompt(String ask, boolean newline) {
	return prompt(ask, newline, "");
    }
          
    private static String prompt(String ask, boolean newline, String defaultValue) 
    {
        String ret = defaultValue;
        if (newline) {
	    System.out.println(ask);
	} else {
	    System.out.print(ask);
	}

	try {	      
	    String s = _reader.readLine();
	    if (s.length() > 0) ret = s;
	} catch (Exception e) {
	    e.printStackTrace();
	}

        return ret;
    }

}            
